import java.applet.Applet;
import java.util.*;
import java.awt.Point;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Event;
import java.awt.Color;

class Room extends Panel 
{
private Vector users;
private boolean mouseDown=false;
private int oldx, oldy;
private TH parent ;
private History history;
private THUser activeUser;
private Date today = null;

public Room(TH p)
  {
    validate();
    users = new Vector(2,1);
    activeUser = null;
    parent =p;
  }

public Applet getApplet() {
  return (Applet)parent;
}

public void addHistory(History hist) {
  history = hist;
}


public synchronized void paint( Graphics g)
  {
    for(int i=0; i<users.size(); i++) {
      THUser u = (THUser)users.elementAt(i);
      u.paint(g);
    }
    Dimension d = size();
    g.setColor( Color.red);
    g.drawRect(0,0,d.width-1, d.height-1);
  }

public synchronized void paintUsers( Graphics g, int x, int y, int w, int h) {
  Rectangle rect = new Rectangle( x, y, w, h);
  Graphics gg = g.create();
  gg.clipRect( x, y, w, h);
  for(int i=0; i<users.size()-1; i++) {
    THUser u = (THUser)users.elementAt(i);
    u.paintArea( gg, rect);
  }
}
 
public synchronized  void updateBorder( Graphics g) {
  g.setColor( Color.red);
  g.drawRect(0,0,size().width-1, size().height-1);
}

public synchronized  void userOnTop( THUser u) {
  users.removeElement( u);
  users.addElement( u);
}

public synchronized  Point addUser( int id, String name) {
  double w = size().width - 80;
  if (w <= 0) w = 100;
  double h = size().height - 80;
  if (h <= 0) h = 200;

  int x = (int) (w * Math.random());
  int y = (int) (h * Math.random());
System.out.println ("x = "+x+" y="+y);
  Point pos = new Point(x,y);
  THUser u = new THUser( this, id, name, pos);
  users.addElement(u);
  u.paint( getGraphics());
  history.addEvent( u.getName() + " has come");
  return pos;
}

public synchronized  Point addUser( int id, String name, int x, int y) {
  double w = size().width - 80;
  if (w <= 0) w = 100;
  double h = size().height - 80;
  if (h <= 0) h = 200;

  Point pos = new Point(x,y);
  THUser u = new THUser( this, id, name, pos);
  users.addElement(u);
  u.paint( getGraphics());
  history.addEvent( u.getName() + " has come");
  return pos;
}

public synchronized  void newFace( int id, String sURL) {
  for(int i=0; i<users.size(); i++) {	 
    THUser u = (THUser)users.elementAt(i);
    if(u.getID() == id) {
      u.changeFace( sURL);
      u.paint( getGraphics());
      history.addEvent( u.getName() + " got a new look");
      return;
    }
  }
}

public synchronized  THUser getUser(int id){
  int i;
  for(i=0; i<users.size(); i++) {	 
    THUser u = (THUser)users.elementAt(i);
    if(u.getID() == id) break;
  }
  return (THUser)users.elementAt(i);
}

public synchronized  THUser getUser(String uname){
  int i;
  for(i=0; i<users.size(); i++) {	 
    THUser u = (THUser)users.elementAt(i);
    if(uname.equals(u.getName())) return (THUser)users.elementAt(i);
  }
  return null;
}
public synchronized  void userRemove(int i)
  {
    THUser u = getUser(i);
    history.addEvent( u.getName() + " has left");
    users.removeElement(u);
    repaint();
  }

  synchronized public void userSaid(int i, String str) { 
    THUser u = getUser(i);
    userOnTop( u);
    if (!mouseDown || u != activeUser)
      u.paint( getGraphics());
    u.say(str);
    today = new Date();
    history.addEvent( "("+today.getHours()+":"+today.getMinutes()+") "+
	u.getName() + ": " + str);
  }

 synchronized public boolean mouseDown(Event event,int x,int y) {

    if (parent.myself.mouseWithin(x,y)) {
        userOnTop(parent.myself);
        oldx = x;
        oldy = y;
        mouseDown = true;
        activeUser = parent.myself;
        activeUser.setInMove( true);
        activeUser.clear( getGraphics());
        updateBorder( getGraphics());
	parent.sendmsg(parent.username+"^0"+x+","+y);
    }
    return true;
  }

  synchronized public boolean mouseDrag(Event event, int x, int y) {
    if (mouseDown) {
     	int dx = x - oldx;
      	int dy = y - oldy;	
      	if(Math.abs(dx) < 5 && Math.abs(dy) < 5) return true;
      	Dimension d=size();
      	Rectangle userArea = activeUser.getGlowaArea();
      	userOnTop( activeUser);
      	if (userArea.x + dx <= 0 ) x = oldx - userArea.x + 1;
      	if (userArea.y + dy <= 0 ) y = oldy - userArea.y + 1;
      	if (userArea.x + userArea.width + dx  >= d.width-2)  x = oldx + d.width  - (userArea.x + userArea.width)  - 2;
      	if (userArea.y + userArea.height + dy >= d.height-2) y = oldy + d.height - (userArea.y + userArea.height) - 2;
			
      	activeUser.move( getGraphics(), x-oldx, y-oldy);
      	oldx = x;
      	oldy = y;
	//parent.sendmsg(parent.username+"^1"+x+","+y);
     }
     return true;
  }

  synchronized public boolean mouseUp(Event event,int x,int y) {
    if (parent.myself.mouseWithin(x,y)) {
	mouseDown = false;
    	activeUser.moveEnd();
    	activeUser.setInMove( false);
    	activeUser.paint( getGraphics());
    	parent.sendmsg(parent.username+"^2"+x+","+y);
    }
    return true;
  }	
}
